;------------------------------------------------------------------------------
; *** CP/M BIOS pre Consul 2717 Zbrojovacek ***
; Ako diskove zariadenie sa pouziva PMD 32-SD.
; Ako disky sa pouzivaju obrazy 8" diskov DZ8 - 500,5 kB

; RB - 28 Oct 2012, 15 Aug 2015, 8 May 2020
;------------------------------------------------------------------------------

CcpLen		equ	0800h		; dlzka CCP
BdosLen		equ	0E00h		; dlzka BDOS
BiosLen		equ	0470h		; dlzka BIOS (MAX)
BiosLenDisk	equ	0380h		; dlzka BIOS (MAX na disku)

BiosAdr		equ	0CB00h		; pociatocna adresa BIOS
BdosAdr		equ	BiosAdr-BdosLen	; pociatocna adresa BDOS
CcpAdr		equ	BdosAdr-CcpLen	; pociatocna adresa CCP

RomSrvAdr	equ	10h		; adresa volania sluzieb v ROM
RomSrvRst	equ	RomSrvAdr/8	; cislo Restartu - RST x
LocalStack	equ	0038h		; localny zasobik (rastie k nizsim adresam)

NDRIVES		equ	4		; 4 disky
XBIOS		equ	0		; vstupny bod BIOS
IOBYTE		equ	3		; adresa IOBYTE
CDISK		equ	4		; adresa CDISK
XBDOS		equ	5		; vstupny bod BDOS

CR		equ	0Dh		; CR
LF		equ	0Ah		; LF
EOF		equ	1Ah		; EOF

DmaAdr		equ	0CFB0h		; zdrojova/cielova adresa citania/zapisu
Length		equ	0CFB2h		; dlzka citaneho bloku - 
Disk		equ	0CFB4h		; cislo disku 0 az 3
Stopa		equ	0CFB5h		; cislo stopy 0 az 76
Hlava		equ	0CFB6h		; cislo hlavy 0 alebo 1
Sektor		equ	0CFB7h		; cislo sektora 1 az 26
DskPar		equ	0CFB8h		; parametre disku
Result		equ	0CFBCh		; vysledok diskovej operacie

STCSL		equ	0CFF5h		; priznaky klavesnice
OLDKY		equ	0CFF6h		; stav brany a casovaca

; vektory sluzieb v ROM
CSI		equ	03h		; vstupny bod CSI
CPSLCK		equ	06h		; vstupny bod CPSLCK
COUT		equ	09h		; vstupny bod COUT
ICON		equ	0Ch		; vstupny bod ICON
SREAD		equ	15h		; vstupny bod READ
SWRITE		equ	18h		; vstupny bod WRITE

; systemova 8255
SYS55B		equ	0F5h		; port citania klavesnice
SYS55M		equ	0F7h		; riadiaci port
SRAM		equ	0Dh		; pripojenie RAM
SROM		equ	0Ch		; pripojenie ROM

; PIO 8255
PIO55B		equ	4Dh		; Port B - vystup v MODE 1 - Tlaciaren
PIO55C		equ	4Eh		; Port C - handshake signaly
PIO55M		equ	4Fh		; riadiaci port 8255
PIO55I		equ	0C4h		; inicializacny byte 8255

; PIT 8253
PIT53T1		equ	5Dh		; port casovaca 1
PIT53T2		equ	5Eh		; port casovaca 2
PIT53M		equ	5Fh		; riadiaci port
PIT53I1		equ	74h		; inicializacny byte casovaca 1
PIT53I2		equ	0B4h		; inicializacny byte casovaca 2
PIT53C2		equ	80h		; zachytenie stavu casovaca 2

; 9 vektorov sluzieb ovladaca PMD 32-SD
ReadByte	equ	905Ah
SendCommand	equ	905Dh
SendByte	equ	9060h
SndCrcRdAckErr	equ	9063h
ReadAckErr	equ	9066h
WaitErrT15	equ	9069h
WaitErrT	equ	906Ch
SndCrcRdAck	equ	906Fh
ReadCheckCrc	equ	9072h

; flag, ci sa ma po starte spustit CD.COM
F_CDCOM		equ	0

;------------------------------------------------------------------------------

		org	BiosAdr

		jmp	CBOOT
		jmp	WBOOT
		jmp	CONST
		jmp	CONIN
		jmp	CONOUT
		jmp	LIST
		jmp	PUNCH
		jmp	READER
		jmp	HOME
		jmp	SELDSK
		jmp	SETTRK
		jmp	SETSEC
		jmp	SETDMA
		jmp	READ
		jmp	WRITE
		jmp	LISTST
		jmp	SECTRN

;------------------------------------------------------------------------------
; definicie pre PMD 32-SD (pouziva napr. CD.COM)
		db	"32"		; znacka pre vyhladanie tabulky
		db	4*16+0		; 4 disky od A:
		db	91		; 256 - ("3" + "2" + (4 * 16 + 0))
		db	"DZ8"		; defaultna pripona sboru
		dw	Paging		; adresa strankovacej rutiny
		dw	ReadByte	; 9 vektorov sluzieb ovladaca PMD 32-SD
		dw	SendCommand
		dw	SendByte
		dw	SndCrcRdAckErr
		dw	ReadAckErr
		dw	WaitErrT15
		dw	WaitErrT
		dw	SndCrcRdAck
		dw	ReadCheckCrc

;------------------------------------------------------------------------------
; Zahlavie diskovych parametrov
DPH:
DPH0:		dw	XLT,0		; adresa prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adresa buffra, adresa diskovych parametrov
		dw	CSV0,ALV0	; adresa zony kontr. suctov a bitovej alok. mapy
DPH1:		dw	XLT,0		; adresa prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adresa buffra, adresa diskovych parametrov
		dw	CSV1,ALV1	; adresa zony kontr. suctov a bitovej alok. mapy
DPH2:		dw	XLT,0		; adresa prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adresa buffra, adresa diskovych parametrov
		dw	CSV2,ALV2	; adresa zony kontr. suctov a bitovej alok. mapy
DPH3:		dw	XLT,0		; adresa prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adresa buffra, adresa diskovych parametrov
		dw	CSV3,ALV3	; adresa zony kontr. suctov a bitovej alok. mapy

; tabulka diskovych parametrov
; 26 sektorov na stopu, 77 stop, 2 strany, 500,5 kB (ako 8" obojstranny disk)
; 2kB alokacny blok, 247 alokacnych blokov, kapacita 494 kB
; 128 adresarovych poloziek, 2 systemove stopy
DPB:		dw	26		; SPT - pocet logickych sektorov na stopu
		db	4		; BSH - posun bloku
		db	15		; BLM - maska bloku
		db	1		; EXM - maska rozsirenia
		dw	246		; DSM - velkost disku-1
		dw	127		; DRM - velkost adresara-1
		db	192		; AL0 - alokacna maska adresara
		db	0		; AL1
		dw	32		; CKS - velkost pola kontrolnych suctov 
		dw	2		; OFF - pocet systemovych stop

; prekladova tabulka
XLT:		db	1,7,13,19,25,5,11,17,23,3, 9,15,21
		db	2,8,14,20,26,6,12,18,24,4,10,16,22

;------------------------------------------------------------------------------
StartTxt:	db	"52K CP/M V2.2 for PMD 32-SD",CR,LF
		db	"RM-TEAM 2012, 2015, 2020",CR,LF,0

BootTxt:	db	CR,LF,"Boot?",0

	if	F_CDCOM
CCPCommand:	db	4,"CD *",0
CCPCommandLen	equ	$-CCPCommand
	endif

;------------------------------------------------------------------------------
; Studeny start systemu
CBOOT:		di
		; inicializacia casovacov pre AutoRepeat klavesov
		mvi	a,PIT53I1	; casovac 1, rezim 2, 2 slabiky, binarne
		out	PIT53M
		mvi	a,PIT53I2	; casovac 2, rezim 2, 2 slabiky, binarne
		out	PIT53M
		xra	a
		out	PIT53T1		; - hodiny pre 2. casovac - ~31Hz
		out	PIT53T1
		out	PIT53T2		; "pocitadlo" 31Hz tickov
		out	PIT53T2

		; inicializacia PIO pre PMD 32-SD a tlaciaren
		mvi	a,PIO55I	; SK A: MOD 2, A-IN/OUT, CH-OUT
		out	PIO55M		; SK B: MOD 1, B-OUT
		mvi	a,5		; 1->C2 povolenie prerusovacieho
		out	PIO55M		;   signalu INTR pre vystup B

		; prekopirovanie rutiny pre volanie sluzieb v ROM
		lxi	h,RomSrvX
		lxi	d,RomSrv
		mvi	c,RomSrvY-RomSrvX
		call	CopyMem

		mvi	l,ICON		; vstupny bod ICON
		rst	RomSrvRst	; - inicializacia klavesnice a obrazovky

		lxi	h,StartTxt	; adresa uvodnej hlasky
		call	Print0		; zobraz

		mvi	a,95h		; I/O BYTE
		sta	IOBYTE
		xra	a		; CDISK = 0 => A: / User: 0
		sta	CDISK

	if	F_CDCOM
		lxi	h,CCPCommand	; priprav prikaz "CD *"
		lxi	d,CcpAdr+7	; do prikazoveho riadku CCP
		mvi	c,CCPCommandLen
		call	CopyMem

		jmp	WBootCont
	else
		jmp	WBootContX
	endif

;------------------------------------------------------------------------------
BootError:	lxi	h,BootTxt	; adresa chybovej hlasky
		call	Print0		; zobraz
		call	CONIN		; pockaj na klaves

;------------------------------------------------------------------------------
; Teply start systemu.
WBOOT:		lxi	sp,0080h	; nastav adresu zasobnika
		mvi	c,0		; disk A:
		CALL	SELDSK		; vyber disk
		lxi	b,CcpAdr	; zaciatocna adresa CCP
		lxi	h,25*128-1	; dlzka CCP a BDOS v 1. sys. stope
		call	SetLenDma	; uloz DMA a LEN
		mvi	c,0		; stopa 0
		call	SETTRK
		mvi	c,2		; CCP je od sektora 2
		call	SETSEC
		call	READ		; precitaj prvu systemovu stopu
		jnz	BootError	; doslo ku chybe, skoc zobrazit "BOOT?"
		lxi	b,CcpAdr+25*128	; adresa pokracovania CCP a BDOS
		lxi	h,19*128-1	; dlzka CCP a BDOS v 2. sys. stope
		call	SetLenDma	; uloz DMA a LEN
		mvi	c,1		; stopa 1
		call	SETTRK
		mvi	c,1		; od sektora 1
		call	SETSEC
		call	READ		; precitaj zbytok CCP a BDOS
		jnz	BootError	; doslo ku chybe, skoc zobrazit "BOOT?"
		lda	BdosAdr+8	; jednoduchy test, ci je system spravne
		cpi	BdosAdr>>8	; nacitany
		jnz	BootError	; nie je, skoc zobrazit "BOOT?"
WBootContX:	xra	a		; vynuluj preventivne pocet zadanych
		sta	CcpAdr+7	; znakov z konzoly v CCP

WBootCont:	lxi	b,0080h		; defaultne DMA a LEN
		lxi	h,007Fh
		call	SetLenDma	; uloz DMA a LEN
		mvi	a,0C3h		; JMP
		STA	XBIOS
		lxi	h,BiosAdr+3	; adresa tepleho startu BIOSu
		shld	XBIOS+1
		sta	XBDOS		; JMP
		lxi	h,BdosAdr+6	; adresa skoku do BDOSu
		shld	XBDOS+1
		xra	a		; vynuluj kod posledneho znaku z konzoly
		sta	LastKey
		lda	CDISK		; CDISK
		mov	c,a		; do C
		jmp	CcpAdr		; skoc do CCP

;------------------------------------------------------------------------------
; CONST - stav konzoly CON:
; I: -
; O: A = 0 - nebola stlacena klavesa
;    A = 0FFh - bola stlacena klavesa
; M: vsetky
;------------------------------------------------------------------------------
CONST:		lda	LastKey		; bola stlacena nejaka klavesa?
		ora	a
		jnz	Const1		; ano, skoc dopredu
		call	SaveSP		; nastav novu adresu SP
		call	ScanKbd		; otestuj klavesnicu
		call	RestoreSP	; obnov povodnu adresu SP
		mov	a,c		; vezmi kod stlacenej klavesy
		sta	LastKey		; a uloz
		ora	a		; platny kod?
		rz			; nie, vrat sa
Const1:		ori	0FFh		; flag: bola stlacena klavesa
		ret

;------------------------------------------------------------------------------
; Test klavesnice.
; I:
; O: Z - nic nebolo stlacene
;    NZ - stlacena klavesa, A=C=kod klavesy
; M: vsetky
;------------------------------------------------------------------------------
ScanKbd:	mvi	a,PIT53C2	; zachytenie obsahu citaca 2
		out	PIT53M
		in	PIT53T2
		mov	e,a
		in	PIT53T2
		mov	d,a		; do DE
		lxi	h,OLDKY		; adresa so stavom portu klavesnice
		in	SYS55B		; stav portu klavesnice
		ani	7Fh		; odmaskuj platne bity
		cmp	m		; doslo ku zmene?
		jnz	ScanKbd2	; ano, skoc dalej
		lhld	KeyRepTim	; nedoslo, predosla hodnota citaca do HL
		mov	a,l		; odpocitaj novu hodnotu
		sub	e
		mov	l,a		; odpamataj rozdiel
		mov	a,h
		sbb	d		; dlhy cas od posledneho testu?
		jnz	ScanKbd3	; ano, novy test
		lda	KeyRepDel	; cas opakovania klavesy
		cmp	l		; uz nastal ?
		jc	ScanKbd3	; ano skoc
		xra	a		; nie, vrat sa
		mov	c,a
		ret

ScanKbd2:	mvi	m,0		; vynuluj stav portu
		mvi	a,15		; cas do opakovania klavesy asi 480 ms
		db	1		; LXI B - preskoc nasledujucu instrukciu
ScanKbd3:	mvi	a,2		; cas opakovania klavesy 64 ms
		sta	KeyRepDel	; uloz novy cas
		xchg
		shld	KeyRepTim	; uloz aktualnu hodnotu citaca
		mvi	a,2		; timeout pre test klavesnice
		mov	b,a
		sta	OLDKY+1
		mvi	l,CSI		; vstupny bod CSI
		rst	RomSrvRst	; - citanie z klavesnice
		rz
		mvi	l,CPSLCK	; vstupny bod CPSLCK
		rst	RomSrvRst	; - korekcia kodu klavesu
		ret

;------------------------------------------------------------------------------
; CONIN - vstup znaku z konzoly CON:
; I: -
; O: A = kod klavesy/znaku z konzoly
; M: vsetky
;------------------------------------------------------------------------------
CONIN:		call	CONST		; cakaj na stlacenie klavesy
		jz	CONIN
		lxi	h,LastKey	; adresa s kodom klavesy
		mov	a,m		; vezmi znak
		mvi	m,0		; vynuluj
		ora	a		; je to kod do 07Fh ?
		rp			; ano, vrat sa
		cpi	0A0h		; su to znaky s diakritikou ?
		rnc			; ano, vrat sa
		ani	07Fh		; odmaskuj riadiace kody
		ret

;------------------------------------------------------------------------------
; CONOUT - vystup znaku na konzolu CON:
; I: C = znak posielany na konzolu
; O: -
; M: vsetky
;------------------------------------------------------------------------------
CONOUT:		call	SaveSP		; nastav novu adresu SP
		mvi	l,COUT		; vstupny bod COUT
		rst	RomSrvRst	; - vystup znaku na obrazovku
		call	RestoreSP	; obnov povodnu adresu SP
		ret

;------------------------------------------------------------------------------
; LIST - vystup znaku na LST:
; I: C = tlaceny znak
; O: -
; M: AF
;------------------------------------------------------------------------------
LIST:		call	LISTST		; cakaj na uvolnenie tlaciarne
		jz	LIST
		mov	a,c		; odosli znak na tlaciaren
		out	PIO55B
		ret

;------------------------------------------------------------------------------
; LISTST - stav zariadenia LST:
; I: -
; O: A = 0 - tlaciaren nie je pripravena
;    A = 0FFh - tlaciaren je pripravena
; M: AF
;------------------------------------------------------------------------------
LISTST:		in	PIO55C		; precitaj stav portu tlaciarne
		rrc			; INTRB=1 = BUSY -> CY
		sbb	a		; A = 0 alebo 0FFh
		ret

;------------------------------------------------------------------------------
; READER - citanie znaku z logickeho zariadenia vstupu
; - nevyuzite, vrati vzdy EOF
; I: -
; O: A=1Ah
; M: -
;------------------------------------------------------------------------------
READER:		mvi	a,EOF		; vrat vzdy EOF

;------------------------------------------------------------------------------
; PUNCH - vystup znaku na logicke zariadenie znakoveho vystupu
; - nevyuzite
; I: C = znak
; O: -
; M: -
;------------------------------------------------------------------------------
PUNCH:		ret

;------------------------------------------------------------------------------
; SELDSK - vyber disku
; I: C = cislo disku
; O: HL = adresa diskovych parametrov
;         HL = 0000, ak disk neexistuje
; M: HL, DE, AF
;------------------------------------------------------------------------------
SELDSK:		lxi	h,0		; 0 - neplatny disk
		mov	a,c		; cislo disku
		cpi	NDRIVES
		rnc			; mimo rozsah
		sta	Disk		; uloz cislo disku
		add	a		; vypocitaj offset k aktualnemu disku
		add	a
		add	a
		add	a
		mov	l,a
		lxi	d,DPH		; adresa zahlavia diskovych parametrov
		dad	d		; pripocitaj zakladnu adresu
		ret

;------------------------------------------------------------------------------
; HOME - hlava vybraneho disku na 0. stopu
; I: -
; O: -
; M: -
;------------------------------------------------------------------------------
HOME:		mvi	c,0

;------------------------------------------------------------------------------
; SETTRK - nastavenie stopy na vybranom disku
; I: C=cislo logickej stopy
; O: -
; M: -
;------------------------------------------------------------------------------
SETTRK:		mov	a,c
		sta	ReqTrack
		ret

;------------------------------------------------------------------------------
; SETSEC - nastavenie sektora
; I: C=fyzicke cislo sektoru
; O: -
; M: -
;------------------------------------------------------------------------------
SETSEC:		mov	a,c
		sta	Sektor
		ret

;------------------------------------------------------------------------------
; Nastavenie dlzky dat (LEN) a adresy v pamati (DMA)
; I: HL=dlzka dat, BC=DMA adresa
; O: -
; M: HL
;------------------------------------------------------------------------------
SetLenDma:	shld	Length		; uloz dlzku dat

;------------------------------------------------------------------------------
; SETDMA - nastavenie DMA adresy
; I: BC=DMA adresa
; O: -
; M: HL
;------------------------------------------------------------------------------
SETDMA:		mov	l,c		; uloz DMA
		mov	h,b
		shld	DmaAdr
		ret

;------------------------------------------------------------------------------
; READ - precitanie zvoleneho sektora (skupiny sektorov)
; I: -
; O: A=0 - OK, A<>0 - chyba
; M: vsetky
;------------------------------------------------------------------------------
READ:		mvi	a,SREAD		; vektor ROM sluzby - READ
		db	33		; LXI H - preskoc nasledujucu instrukciu

;------------------------------------------------------------------------------
; WRITE - zapis do zvoleneho sektora (skupiny sektorov)
; I: C=0 - normalny zapis, C=1 - zapis do adresara,
;    C=2 - prvy zapis do nealokovaneho bloku
; O: A=0 - OK, A<>0 - chyba
; M: vsetky
;------------------------------------------------------------------------------
WRITE:		mvi	a,SWRITE	; vektor ROM sluzby - WRITE
		sta	OperRW+1
		lda	ReqTrack	; cislo pozadovanej stopy
		ora	a		; CY=0
		rar			; vydel dvomi -> CY je cislo hlavy
		sta	Stopa		; a do syst. premennej
		mvi	a,0		; priprav cislo hlavy
		ral
		sta	Hlava		; a uloz do syst. premennej
		call	SaveSP		; nastav novu adresu SP
OperRW:		mvi	l,0		; precitaj/zapis sektor(y)
		rst	RomSrvRst
		call	RestoreSP	; obnov povodnu adresu SP
		mvi	a,0		; nastav chybovu hodnotu podla CY
		aci	0		
		ret

;------------------------------------------------------------------------------
; SECTRN - prepocet logickeho/fyzickeho sektora
; I: C=logicke cislo sektora, DE=adresa prekladovej tabulky
; O: HL=fyzicke cislo sektora
; M: AF, B, DE, HL
;------------------------------------------------------------------------------
SECTRN:		mvi	b,0
		xchg
		dad	b
		mov	l,m
		mov	h,b
		ret

;------------------------------------------------------------------------------
; Rutina nastavi lokalny zasobnik pod adresu 0038.
; I: -
; O: SP=0038h
; M: CY, SP, HL, DE
;------------------------------------------------------------------------------
SaveSP:		pop	d		; navratova adresa do DE
		lxi	h,0		; adresa zasobnika do HL
		dad	sp
		shld	OldSP+1		; a odpamataj
		lxi	sp,LocalStack	; nova adresa zasobnika
		xchg			; nepriamo sa vrat
		pchl

;------------------------------------------------------------------------------
; Rutina vrati zasobnik na povodnu adresu.
; I: -
; O: -
; M: SP, HL
;------------------------------------------------------------------------------
RestoreSP:	pop	h		; navratova adresa do HL
OldSP:		lxi	sp,0		; povodny zasobnik
		pchl			; nepriamo sa vrat

;------------------------------------------------------------------------------
; Rutina pre zobrazenie textu ukonceneho 0.
; I: HL = adresa textu ukonceneho 0
; O: -
; M: vsetky
;------------------------------------------------------------------------------
Print0:		mov	a,m		; vezmi znak
		ora	a		; je to 0 ?
		rz			; ano, vrat sa
		push	h		; odpamataj ukazatel
		mov	c,a		; uloz do C
		mvi	l,COUT		; vstupny bod COUT
		rst	RomSrvRst	; zobraz znak
		pop	h		; obnov ukazatel
		inx	h		; posun ukazatel
		jmp	Print0		; pokracuj v slucke

;------------------------------------------------------------------------------
; Kopia casti pamate.
; I: HL=zdrojova adresa, DE=cielova adresa, c=pocet kopirovanych bytov
CopyMem:	mov	a,m
		stax	d
		inx	h
		inx	d
		dcr	c
		jnz	CopyMem
		ret

;------------------------------------------------------------------------------
; Volanie sluzieb z ROM - rutina sa presuva na adresu RomSrvAdr
; a vola RST RomSrvRst
; I: L = nizsi byte adresy sluzby v ROM
;------------------------------------------------------------------------------
RomSrvX:
		phase	RomSrvAdr
RomSrv:		mvi	h,90h		; vyssi byte adresy sluzby v ROM
		mvi	a,SROM		; nastrankuj ROM
		out	SYS55M
		call	CallSrv		; zavolaj sluzbu
		mvi	a,SRAM		; nastrankuj RAM
		out	SYS55M
		ret

CallSrv:	pchl			; skok na adresu sluzby
		dephase
RomSrvY:

;------------------------------------------------------------------------------
; Volanie rutin ROM z externych programov.
; I: HL=adresa rutiny
Paging:		push	psw
		mvi	a,SROM		; nastrankuj ROM
		out	SYS55M
		pop	psw
		call	CallSrv		; zavolaj sluzbu
		push	psw
		mvi	a,SRAM		; nastrankuj RAM
		out	SYS55M
		pop	psw
		ret

;------------------------------------------------------------------------------

BiosCodeEnd	equ	$

		if	(BiosCodeEnd > (BiosAdr + BiosLenDisk))
		  error "BIOS je prilis dlhy!!!"
_Free1		  equ	0FFFFh
		else
_Free1		  equ	((BiosAdr + BiosLenDisk) - BiosCodeEnd)
		endif

;------------------------------------------------------------------------------

ReqTrack:	db	0		; pozadovana stopa
LastKey:	db	0		; posledny znak z konzoly
KeyRepDel:	db	0		; pocitadlo pre opakovanie klavesy
KeyRepTim:	dw	0		; hodnota citaca pre opakovanie klavesy

DIRBUF:		ds	128		; DIRBUF - adr. buf. pre 1 zazn. adres.
ALV0:		ds	31		; ALV - adr. obl. alokac. vekt. disku A
ALV1:		ds	31		; ALV - adr. obl. alokac. vekt. disku B
ALV2:		ds	31		; ALV - adr. obl. alokac. vekt. disku C
ALV3:		ds	31		; ALV - adr. obl. alokac. vekt. disku D
CSV0:		ds	32		; CSV - adr. oblasti kontr. suctov A
CSV1:		ds	32		; CSV - adr. oblasti kontr. suctov B
CSV2:		ds	32		; CSV - adr. oblasti kontr. suctov C
CSV3:		ds	32		; CSV - adr. oblasti kontr. suctov D

;------------------------------------------------------------------------------

		if	($ > (BiosAdr + BiosLen))
		  error "Data BIOSu prekracuju maximum!!!"
_Free2		  equ	0FFFFh
		else
_Free2		  equ	((BiosAdr + BiosLen) - $)
		  org	BiosCodeEnd
		  db	_Free1 dup 0E5h
		endif

		if	(_Free1 < 0FFFFh && _Free2 < 0FFFFh)
		  if	(_Free1 < _Free2)
_Free		    equ	_Free1
		  else
_Free		    equ	_Free2
		  endif
		else
_Free		  equ	0
		endif
;------------------------------------------------------------------------------
