;------------------------------------------------------------------------------
; Diskovy ovladac PMD 32 pre C2717 namiesto povodneho ovladaca 8" mechanik.
; Ovladac je urceny a usity pre povodne verzie EPROM s koncovym oznacenim _01.
; Meni sa 1 EPROM s oznacenim 513_01. Nova ma oznacenie 513_32.

; RB - 8 Nov 2009 / 8 Jan 2011 / 28 Oct 2012
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------

DMA		equ	0CFB0h		; adresa sektora v pamati
LEN		equ	0CFB2h		; dlzka sektora - 1
DISK		equ	0CFB4h		; cislo disku 0, 1, 2, 3
STOPA		equ	0CFB5h		; cislo stopy/cylindra
HLAVA		equ	0CFB6h		; cislo hlavy 0, 1
SEKTOR		equ	0CFB7h		; cislo sektora
DSKPAR		equ	0CFB8h		; parametre diskety - N, EOT, GPL, DTL
RSLT		equ	0CFBCh		; hlasenia FDC 8272 - vysledok FDC prikazu
RSLT_C		equ	RSLT+0		; cylinder
RSLT_H		equ	RSLT+1		; hlava
RSLT_R		equ	RSLT+2		; sektor
RSLT_N		equ	RSLT+3		; velkost sektora
ROUTINE		equ	0CFFEh		; adresa rutiny pre zapis/citanie

XDma		equ	0080h		; default DMA addresa
XBoot		equ	0080h		; adresa kam sa nahrava BOOT sektor
XLen		equ	80h		; dlzka sektora
XIntr		equ	38h		; adresa prerusenia
Retry		equ	5		; pocet pokusov citania/zapisu sektora

EAdr		equ	9000h		; adresa patchovanej EPROM
Ent1Adr		equ	900Fh		; adresa povodnych vstupnych bodov
Ent2Adr		equ	905Ah		; adresa novych vstupnych bodov
PatchB		equ	9581h		; zaciatok patchovaneho kodu
PatchE		equ	979Ch		; koniec patchovaneho kodu
ROMCRC		equ	979Ch		; CRC pre ROM

; rutiny v zakladnom monitore
MEMIN		equ	81B0h		; prenos obsahu pamati
ZPRAVA		equ	818Dh		; vypis spravy do dialogoveho riadku
BEEP		equ	88A3h		; pipnutie
STOP		equ	8C74h		; Test klavesy STOP
MONZ		equ	8FF1h		; navrat do Monitora - teply start

; systemova 8255
SYS55M		equ	0F7h		; systemova 8255 - riadiaci port
SRAM		equ	0Dh		; pripojenie RAM
SROM		equ	0Ch		; pripojenie ROM

; 8255 pre PMD 32
PIO55A		equ	4Ch		; Port A - obojstranne DATA v MODE 2
PIO55B		equ	4Dh		; Port B - vystup v MODE 1 - Tlaciaren
PIO55C		equ	4Eh		; Port C - handshake signaly
PIO55M		equ	4Fh		; riadiaci port 8255
PIO55I		equ	0C4h		; inicializacny byte 8255
ENAIN		equ	09h		; povelenie vstupu A
DISIN		equ	08h		; zakazanie vstupu A
ENAOUT		equ	0Dh		; povolenie vystupu A
DISOUT		equ	0Ch		; zakazanie vystupu A
INTRA		equ	08h		; maska INTRA
PRESENT		equ	0AAh		; prezentacny byte
ACK		equ	33h		; ACK z PMD 32-SD

;------------------------------------------------------------------------------

		org	EAdr		; vlozenie povodnej 2kB binarky

		binclude "d:\8bit\PMD-85\projects\c2717-pmd32\513_01"

;------------------------------------------------------------------------------

		org	Ent1Adr		; nove vstupne body pre povodne sluzby

		jmp	Recalibrate
		jmp	SeekTrack
		jmp	ReadSector
		jmp	WriteSector
		jmp	Specify
		jmp	SenseDrive
		jmp	SenseInt
		jmp	RecalibrateCmd
		jmp	SeekCmd
		jmp	ReadIdCmd
		jmp	ReadCmd
		jmp	WriteCmd
		jmp	FormatCmd
		jmp	ResultCmd
		jmp	MoveIntr

		org	Ent2Adr		; vstupne body novych sluzieb pre obsluhu PMD 32 

		jmp	ReadByte
		jmp	SendCommand
		jmp	SendByte
		jmp	SndCrcRdAckErr
		jmp	ReadAckErr
		jmp	WaitErrT15
		jmp	WaitErrT
		jmp	SndCrcRdAck
		jmp	ReadCheckCrc

;------------------------------------------------------------------------------

		org	PatchB		; zaciatok patchu 1 

;------------------------------------------------------------------------------

Boot:		call	CheckDevice	; zisti, ci je pripojena disketova jednotka
		rc			; nie, vrat sa

		lxi	sp,XDma

		call	MoveIntr	; presun kod pre transfer dat do RAM

		lxi	d,MBoot		; zobraz hlasku ++ Boot ++
		call	ZPRAVA
		call	BEEP

Boot2:		lxi	h,XBoot		; precitaj BOOT sektor
		mvi	a,"B"		; odosli povel
		call	SendCommand
		call	RdSector2	; precitaj sektor
		jc	Boot3		; doslo ku chybe, skoc dalej
		mvi	c,6		; test signatury v BOOTe
		lxi	d,XBoot+3
		lxi	h,C2717
Boot4:		ldax	d
		xra	m
		jnz	Boot3		; nie je to systemova disketa C2717
		inx	h
		inx	d
		dcr	c
		jnz	Boot4
		call	XBoot		; skoc vykonat BOOT kod

Boot3:		call	STOP		; test klavesy STOP
		jnz	Boot2		; nebola stlacena, opakuj
		jmp	MONZ		; navrat do Monitora

MBoot:		db	"++ Boot ++     "
C2717:		db	"C2717S"

;------------------------------------------------------------------------------
; Precita sektor/y na aktualnej zadanej stope/cylindri 
; I: [DMA], [LEN], [DISK], [STOPA], [HLAVA], [SEKTOR]
; O: CY=1, Z=0 - pri chybe
;    CY=0, Z=1 - OK
; M: vsetky
;------------------------------------------------------------------------------
ReadSector:	lxi	d,RdSector	; adresa rutiny pre citanie sektorov
		mvi	a,0DBh		; IN  PIO55A
		lxi	h,77h*256+PIO55A ; MOV M,A
		jmp	RdWrSector

;------------------------------------------------------------------------------
; Zapise sektor/y na zadanej stope/cylindri 
; I: [DMA], [LEN], [DISK], [STOPA], [HLAVA], [SEKTOR]
; O: CY=1, Z=0 - pri chybe
;    CY=0, Z=1 - OK
; M: vsetky
;------------------------------------------------------------------------------
WriteSector:	lxi	d,WrSector	; adresa rutiny pre zapis sektorov
		mvi	a,7Eh		; MOV A,M
		lxi	h,PIO55A*256+0D3h ; OUT PIO55A
RdWrSector:	sta	TransOper	; uloz instrukcie prenosu
		shld	TransOper+1
		xchg
		shld	ROUTINE		; uloz adresu rutiny
		lhld	STOPA		; prekopiruj pozadovany Cylinder, Hlavu,
		shld	RSLT_C		; Sektor a Velkost sektora do Result pola
		lhld	SEKTOR		; Hlava a Sektor v RSLT sa budu menit
		shld	RSLT_R
		lhld	LEN		; dlzka citaneho/zapisovaneho bloku
		lxi	d,127		; uprav dlzku pre ziskanie poctu celych
		dad	d		; 128 bytovych sektorov
		mov	a,l
		rlc
		mov	a,h
		ral			; pocet sektorov = (LEN + 127) / 128
		mov	b,a		; uloz do B
		lhld	DMA		; DMA
		xchg			; do DE
RdWrSector2:	mvi	c,Retry		; pocet pokusov
RdWrSector5:	push	b		; odpamataj pocitadlo sektorov a opakovani
		push	d		; odpamataj adresu DMA na zasobnik
		call	RdWrSectorCall	; precitaj alebo zapis sektor
		pop	d		; obnov DMA adresu
		pop	b		; obnov pocitadlo pokusov a sektorov
		jnc	RdWrSector6	; bez chyby, skoc dalej
		dcr	c		; dalsie opakovanie ?
		jnz	RdWrSector5
		jmp	RdWrSector4	; chyba, skonci

RdWrSector6:	xchg			; nova DMA adresa do DE
		lxi	h,RSLT_R	; posun na dalsi sektor
		inr	m
		mvi	a,27
		cmp	m		; prechod na druhu stranu?
		jnz	RdWrSector3	; nie, skoc
		mvi	a,1
		mov	m,a		; zacneme znovu od sektora 1
		dcx	h
		xra	m
		mov	m,a		; zmena hlavy
RdWrSector3:	dcr	b		; zniz pocet citanych sektorov
		jnz	RdWrSector2	; opakuj pre vsetky
RdWrSector4:	mvi	a,0
		rar
		ora	a		; nastav NZ, ak bolo CY=1
		rlc
		ret

RdWrSectorCall:	lhld	ROUTINE		; adresa rutiny
		pchl			; skoc na nu

;------------------------------------------------------------------------------
; Precita sektor z disku.
; I: DE=adresa DMA, [DISK], [STOPA], [RSLT_H], [RSLT_R]
; O: CY=1 - pri chybe
;    CY=0 - OK, HL=povodne DMA+128, D=0
; M: vsetky
;------------------------------------------------------------------------------
RdSector:	xchg			; DMA do HL
		mvi	a,"Q"		; "Q" - precitaj sektor
		call	SendCommand	; posli povel
		rc			; navrat pri chybe
		call	SendSecTrk	; posli sektor a stopu
RdSector2:	rc			; navrat pri chybe
		call	SndCrcRdAck	; odosli CRC a pockaj na ACK
		rc			; navrat pri chybe
		mvi	b,0		; vynuluj CRC
		call	Transfer	; precitaj sektor
		jmp	ReadCheckCrc	; skoc precitat CRC

;------------------------------------------------------------------------------
; Zapise sektor na disk.
; I: DE=adresa DMA, [DISK], [STOPA], [RSLT_H], [RSLT_R]
; O: CY=1 - pri chybe
;    CY=0 - OK, HL=povodne DMA+128, D=0
; M: vsetky
;------------------------------------------------------------------------------
WrSector:	xchg			; DMA do HL
		mvi	a,"T"		; "T" - zapis sektor
		call	SendCommand	; posli povel
		rc			; navrat pri chybe
		call	SendSecTrk	; posli sektor a stopu
		rc			; navrat pri chybe
		call	Transfer	; zapis sektor
		jmp	SndCrcRdAck	; odosli CRC a cakaj na potvrdenie

;------------------------------------------------------------------------------
; Rutina pre prenos sektoru z/do PMD 32 do/z pamati. Musi byt umiestnena v RAM,
; pretoze sa modifikuje kod podla smeru operacie a naviac sa odpaja ROM, pri
; samotnom prenose.
; I: HL=adresa DMA, B=CRC
; O: HL=povodne DMA+128, D=0, A=0Ch, B=modifikovane CRC, CY=1
; M: AF, B, D, HL 
;------------------------------------------------------------------------------

UsekIm		equ	005Ch
UsekI		equ	$
		phase	XIntr+3		; 3 byty pre breakpoint DDT, SID, SDT atd.
Transfer:	mvi	a,SRAM		; odpojenie ROM
		out	SYS55M
		mvi	d,128		; pocitadlo 128 bytov
Transfer2:	in	PIO55C		; je na vstupe platny byte (citanie),
		ani	INTRA		;   alebo je vystup volny (zapis) ? 
		jz	Transfer2	; ak nie, cakaj
TransOper:	in	PIO55A		; precitaj byte  ; MOV A,M - byte z pamati
		mov	m,a		; uloz do pamati ; OUT PIO55A - posli
		xra	b		; modifikuj CRC
		mov	b,a		; a vrat ho do B
		inx	h		; posun ukazatel
		dcr	d		; zniz pocitadlo bytov
		jnz	Transfer2	; ak neboli prijate vsetky, vrat sa do slucky
		mvi	a,SROM		; pripojenie ROM
		out	SYS55M
StcRet:		stc
		ret
	if 	$ > UsekIm
		warning "UsekI je prilis dlhy !!!"
	endif
		dephase
UsekIL		equ	$-UsekI

;------------------------------------------------------------------------------
; Presun kodu pre transfer dat do RAM.
; I: -
; O: BC=0, DE=adresa "cakacej slucky"
; M: vsetky
;------------------------------------------------------------------------------
MoveIntr:	call	MEMIN		; rutina presunu
		dw	UsekI		; odkial
		dw	UsekIL		; kolko
		dw	XIntr+3		; kam
		lxi	d,StcRet
		ret

;------------------------------------------------------------------------------
; Posle do PMD 32 cislo sektoru a stopy
; I: [DISK], [STOPA], [RSLT_H], [RSLT_R]
; O: CY=1 - chyba
;    CY=0 - OK, B=A=modifikovane CRC
; M: AF, BC
;------------------------------------------------------------------------------
SendSecTrk:	lda	DISK		; cislo disku do 7. a 6. bitu
		ora	a		;  - 00 = A:, 10 = B:, 01 = C:, 11 = D:
		jz	SendSecTrk2
		mvi	c,0C0h
		xri	3
		jz	SendSecTrk3
		rrc
		rrc
SendSecTrk2:	mov	c,a
SendSecTrk3:	lda	RSLT_R
		dcr	a		; cislo sektora na rozsah 0 - 25
		ora	c		; pripoj disk
		call	SendByte	; posli
		rc
		lda	RSLT_H		; priprav hlavu do CY
		rrc
		lda	STOPA		; stopa
		ral			; * 2 + hlava
		jmp	SendByte	; posli a vrat sa

;------------------------------------------------------------------------------
; Precita byte z PMD 32 a zaroven modifikuje CRC.
; I: -
; O: CY=1 - chyba
;    CY=0 - A=C=precitany byte, B=modifikovane CRC
; M: AF, BC
;------------------------------------------------------------------------------
ReadByte:	push	d
		call	IsIntra		; je platny byte na vstupe?
		pop	d
		rc			; nie, vrat sa s chybou
		in	PIO55A		; precitaj byte
		mov	c,a		; odloz si ho do C
		xra	b		; modifikuj CRC
		mov	b,a		; a uloz ho naspat do B
		mov	a,c		; vrat nacitany byte do A
		ret

;------------------------------------------------------------------------------
; Posle povel do PMD 32. Pred poslanim povelu sa najprv preveri, ci PMD 32
; komunikuje.
; I: A=povel
; O: CY=1 - chyba
;    CY=0 - OK, B=A=modifikovane CRC
; M: AF, B
;------------------------------------------------------------------------------
SendCommand:	mov	b,a		; uloz posielany byte do B
		push	b		; BC na zasobnik
		push	d
		call	IsPresent	; zisti, ci je na zbernici prezentacny byte 0AAh
					; ak ano, odosli naspat prezentacny byte
		pop	d
		pop	b		; obnov BC
		rc			; vrat sa, ak doslo ku chybe
		mov	a,b		; vrat posielany byte do A
		mvi	b,0		; vynuluj CRC
; pokracuj dalej

;------------------------------------------------------------------------------
; Posle byte do PMD 32 a modifikuje priebezne CRC.
; I: A=posielany byte
; O: CY=1 - chyba
;    CY=0 - OK, B=A=modifikovane CRC
; M: AF, B
;------------------------------------------------------------------------------
SendByte:	push	d
		call	IsIntra		; zisti, ci je volny vystup
		pop	d
		rc			; vrat sa, ak doslo ku timeoutu
		out	PIO55A		; posli byte na vystup
		xra	b		; modofikuj CRC
		mov	b,a		; a uloz ho naspat do B
		ret

;------------------------------------------------------------------------------
; Zisti, ci je nastavene INTRA.
; INTRA=1, ak je platny byte na vstupe, alebo bol precitany byte z vystupu
; I: Pri volani od IsIntraD DE=timeout
; O: CY=1 - timeout
;    CY=0 - OK
; M: CY, DE
;------------------------------------------------------------------------------
IsIntra:	lxi	d,205		; timeout asi 5ms
IsIntraD:	push	psw		; uloz AF
IsIntra2:	in	PIO55C		; zisti stav INTRA
		ani	INTRA
		jnz	IsIntra3	; ak je INTRA=1, skoc dopredu
		dcx	d
		mov	a,d
		ora	e
		jnz	IsIntra2	; a ak nie je nulove, vrat sa do slucky
		pop	psw		; doslo k timeoutu, obnov AF
		stc			; nastav CY ako priznak chyby
		ret
IsIntra3:	pop	psw
		ora	a		; vynuluj CY
		ret

;------------------------------------------------------------------------------
; Povoli prerusenie pre vstup na porte A
; I: -
; O: povolene prerusenie pre vstup bytu
; M: A
;------------------------------------------------------------------------------
EnaByteIn:	mvi	a,ENAIN		; 1->C4 povolenie prerusovacieho signalu
		out	PIO55M		; INTRA pre vstup A
		mvi	a,DISOUT	; 0->C6 zakazanie prerusovacieho signalu
		out	PIO55M		; INTRA pre vystup A
		ret

;------------------------------------------------------------------------------
; Povoli prerusenie pre vystup portu A
; I: -
; O: povolene prerusenie pre vystup bytu
; M: A
;------------------------------------------------------------------------------
EnaByteOut:	mvi	a,DISIN		; 0->C4 zakazanie prerusovacieho signalu
		out	PIO55M		; INTRA pre vstup A
		mvi	a,ENAOUT	; 1->C6 povolenie prerusovacieho signalu
		out	PIO55M		; INTRA pre vystup A
		ret

;------------------------------------------------------------------------------
; Zisti, ci je pripojena PMD 32 a ci odpoveda prezentacnym bytom.
; I: -
; O: CY=1 - chyba, timeout, PMD 32 neodpoveda prezentacnym bytom
; M: AF, DE, B
;------------------------------------------------------------------------------
CheckDevice:	mvi	a,PIO55I	; inicializuj GPIO 8255
		out	PIO55M		; SK A: MOD 2, A-in/out,  SK B: MOD 1, B-out
		call	EnaByteIn	; povol prerusenie pre vstup
		lxi	d,10000		; timeout asi 250 ms
		call	IsIntraD	; cakaj na byte
		rc			; timeout, vrat sa
		in	PIO55A		; precitaj prijaty byte
		cpi	PRESENT		; je to prezentacny byte 0AAh ?
		stc			; nastav CY ako priznak chyby
		rnz			; nie je to prezentacny byte 0AAh, vrat sa s CY
		call	EnaByteOut	; povol prerusenie pre vystup
		mvi	a,PRESENT	; posli ako odpoved prezentacny byte 0AAh
		jmp	SendByte

;------------------------------------------------------------------------------
; Kratke zdrzanie
; I: -
; O: kratke zdrzanie asi 90 us
; M: AF
;------------------------------------------------------------------------------
WaitQ:		mvi	a,20
WaitQ1:		dcr	a
		jnz	WaitQ1
		ret

;------------------------------------------------------------------------------
; Zisti, ci na zbernici nie je nahodou prezentacny byte 0AAh, ktory poslala
; disketova jednotka po navrate do "idle" stavu, kedy cakala na povel.
; I: -
; O: CY=1 - chyba
;    CY=0 - OK
; M: AF, DE
;------------------------------------------------------------------------------
IsPresent:	call	EnaByteIn	; povol prerusenie pre vstup
		call	WaitQ		; male zdrzanie asi 90 us
		in	PIO55C
		ani	INTRA		; je platny byte na vstupe?
		jz	EnaByteOut	; ak nie, vrat sa cez povolenie prerusenia pre vystup
IsPresent5:	call	EnaByteIn	; povol prerusenie pre vstup
IsPresent4:	in	PIO55A		; precitaj byte zo vstupu
		cpi	PRESENT		; ak je to prezentacny byte 0AAh
		jz	IsPresent2	; skoc poslat prezentacny byte 0AAh na vystup
		lxi	d,5000		; timeout asi 125 ms
		call	IsIntraD	; cakaj na byte
		rc			; timeout, vrat sa
		jmp	IsPresent4	; ano, skoc precitat byte zo vstupu

IsPresent2:	call	WaitQ		; male zdrzanie asi 90 us
		call	EnaByteOut	; povol prerusenie pre vystup
		mvi	a,PRESENT	; posli na vystup prezentacny byte 0AAh
		out	PIO55A
		lxi	d,5000		; timeout asi 125 ms
		call	IsIntraD	; cakaj na byte
		jc	IsPresent5	; ak nebol prezentacny byte prijaty, skoc otestovat vstup
		ret			; bol prijaty prezentacny byte, vrat sa s vynulovanym CY

;------------------------------------------------------------------------------
; Posle CRC na vystup a pocka na ACK a vysledok povelu.
; I: B=aktualne CRC
; O: CY=1 - chyba
;    CY=0 - OK, A=vysledok povelu
; M: AF, BC, DE
;------------------------------------------------------------------------------
SndCrcRdAckErr:	mov	a,b		; posielane CRC do A
		call	SendByte	; posli na vystup
		rc			; vrat sa, ak doslo ku chybe
ReadAckErr:	call	EnaByteIn	; povol prerusenie na vstupe
		call	ReadByte	; precitaj byte zo vstupu
		rc			; vrat sa, ak doslo ku chybe
		cpi	ACK		; bolo prijate byte ACK?
		stc			; nastav chybovy priznak
		rnz			; vrat sa s chybou, ak to nebolo ACK
WaitErrT15:	lxi	d,3000		; timeout asi 15,5 sec
WaitErrT:	push	d
		call	IsIntra		; je platny byte na vstupe?
		pop	d
		jnc	ReadByte	; ak ano, skoc ho precitat
		dcx	d		; zniz pocitadlo timeoutu
		mov	a,d		; a testuj ho na nulu
		ora	e
		jnz	WaitErrT	; ak nie je nulovy, skoc do slucky
		stc			; nastav priznak chyby
		ret

;------------------------------------------------------------------------------
; Odosle CRC a caka na potvrdenie. Otestuje chybovy kod na nulu.
; I: B=posielane CRC
; O: CY=1 - chyba
;    CY=0 - OK
; M: AF, BC, DE
;------------------------------------------------------------------------------
SndCrcRdAck:	call	SndCrcRdAckErr	; posli CRC a cakaj ACK a vysledok povelu
		rc			; vrat sa, ak doslo ku chybe
		jmp	ReadCheckCrc2	; ak bol druhy byte skutocne 0, je to OK

;------------------------------------------------------------------------------
; Precita byte zo vstupu, predstavujuci CRC
; I: -
; O: CY=1 - chyba CRC alebo timeout
;    CY=0 - CRC OK
; M: AF, BC
;------------------------------------------------------------------------------
ReadCheckCrc:	call	ReadByte	; precitaj CRC zo vstupu
		rc			; vrat sa, ak doslo ku chybe
		mov	a,b		; skontroluj CRC
ReadCheckCrc2:	ora	a
		rz			; OK
		stc			; nastav priznak chyby
		ret

;------------------------------------------------------------------------------
; Vsetky sluzby, okrem ReadSector, WriteSector, vracaju CY=1, ako indikaciu
; vseobecnej chyby.
; I: -
; O: CY=1 
; M: CY
;------------------------------------------------------------------------------
Recalibrate:
SeekTrack:
Specify:
SenseDrive:
SenseInt:
RecalibrateCmd:
SeekCmd:
ReadIdCmd:
ReadCmd:
WriteCmd:
FormatCmd:
ResultCmd:
		stc
		ret

;------------------------------------------------------------------------------

	; koniec patchu
	if 	$ >= PatchE
		error	"Patch je prilis dlhy !!!"
	else
		db	PatchE-$ dup (0FFh)	; zbytok vypln 0FFh
	endif

;------------------------------------------------------------------------------

		org	ROMCRC

CRC:		db	00h		; miesto pre CRC ROM

;------------------------------------------------------------------------------

		end

;------------------------------------------------------------------------------
